@@ -1,6 +1,8 @@
 # encoding: utf-8
+import sys
 from collections import namedtuple
 from threading import RLock
+from typing import List
 
 from mo_future import text
 from mo_imports import export, expect
@@ -80,18 +82,25 @@ def entrypoint(func):
     return output
 
 
-def _verify_whitespace(eng):
-    if eng is None:
+def _verify_whitespace(whi: List):
+    if whi is None:
         return None
-    if isinstance(eng, list):
-        engs = [v for e in eng for v in [_verify_whitespace(e)] if v is not None]
-        if not engs:
+    if isinstance(whi, list):
+        whis = [
+            v
+            for e in whi
+            for v in [_verify_whitespace(e)]
+            if v is not None and v.regex.pattern  # IGNORE NO_WHITESPACE
+        ]
+        if not whis:
             return None
-        whitespace = engs[0]
-        if any(e.id != whitespace.id for e in engs[1:]):
+        whitespace = whis[0]
+        if any(e.id != whitespace.id for e in whis[1:]):
+            # THE TOP-MOST WHITESPACE RULES ARE DIFFERENT FOR EACH ParserElement,
+            # SO PROGRAM DOES NOT KNOW WHICH IS THE MASTER WHITESPACE
             Log.error("must dis-ambiguate the whitespace before parsing")
         return whitespace
-    return eng
+    return whi
 
 
 class Parser(object):
@@ -646,9 +655,7 @@ class ParserElement(object):
         """
         Implementation of & operator - returns `Each`
         """
-        return MatchAll(
-            [self, whitespaces.CURRENT.normalize(other)], whitespaces.CURRENT
-        )
+        return MatchAll([self, whitespaces.CURRENT.normalize(other)])
 
     def __rand__(self, other):
         """
@@ -775,6 +782,20 @@ class _PendingSkip(ParserElement):
         Log.error("use of `...` expression without following SkipTo target expression")
 
 
+def set_parser_names():
+    """
+    ASSIGN parser_name FOR All ParserElements FOUND IN CALLER
+    """
+    frame = sys._getframe(1)
+    items = list(frame.f_locals.items())
+    for k, v in items:
+        try:
+            if isinstance(v, ParserElement) and not v.parser_name:
+                v.parser_name = k
+        except Exception:
+            pass
+
+
 NO_PARSER = (
     ParserElement().set_parser_name("<nothing>")
 )  # USE THIS WHEN YOU DO NOT CARE ABOUT THE PARSER TYPE
