@@ -305,7 +305,16 @@ class And(ParseExpression):
         )
 
     def __regex__(self):
-        return "+", "".join(regex_iso(*e.__regex__(), "+") for e in self.exprs)
+        if self.whitespace is whitespaces.NO_WHITESPACE:
+            return "+", "".join(regex_iso(*e.__regex__(), "+") for e in self.exprs)
+
+        return (
+            "+",
+            regex_iso(*self.whitespace.__regex__(), "+").join(
+                regex_iso(*e.__regex__(), "+")
+                for e in self.exprs
+            ),
+        )
 
     def __str__(self):
         if self.parser_name:
@@ -713,14 +722,14 @@ class MatchAll(ParseExpression):
     __slots__ = []
     Config = append_config(ParseExpression, "min_match", "max_match", "whitespace")
 
-    def __init__(self, exprs, whitespace):
+    def __init__(self, exprs):
         """
         :param exprs: The expressions to be matched
         :param mins: list of integers indincating any minimums
         """
         ParseExpression.__init__(self, exprs)
         self.set_config(
-            whitespace=whitespace,
+            whitespace=whitespaces.CURRENT,
             min_match=[
                 e.parser_config.min_match if isinstance(e, Many) else 1 for e in exprs
             ],
